/******************************************************************************
* Copyright (c) 2011, Linear Technology, Corp.                                *
* Linear Tech Proprietary and Confidential                                    *
*                                                                             *
* Title:                                                                      *
* Author:          Gary Yu                                                    *
* Create Date:     10:08:58 09/09/2015                                        *
* Design Name:                                                                *
* Module Name:                                                                *
* Project Name:                                                               *
* Target Devices:                                                             *
* Tool versions:                                                              *
* Description:                                                                *
*                                                                             *
* Dependencies:                                                               *
*                                                                             *
* Revision 1.0     10:08:58 09/09/2015 Gary Yu                                *
* File Created                                                                *
*                                                                             *
* Additional Comments:                                                        *
*                                                                             *
******************************************************************************/


//----------------------------------------------------------------------------
//---------------------- System Configuration Flow V1.3 ----------------------
//----------------------------------------------------------------------------

// 1)  Turn on FPGA board power (system clock should be present) 
// 2)  Turn on ADC board and capture board (FTDI) power
// 3)  Download FPGA image
// 4)  Configure FTDI as MPSSE mode
// 5)  Generate a reset pulse to FPGA on GPIOL3/BDBUS7 of FTDI    
// 6)  Check ID Register to verify FPGA image is correct    
// 7)  Turn on FPGA reference clock and set ADC as SPI slave by 
//     SPI Configuration Register
// 8)  Reset capture engine by Capture Reset Register  
// 9)  Configure ADC through SPI by FTDI
// 10) Read ADC board ID through I2C interafce accessed by I2C Access Register   
// 11) Generate sysref pulse (SubClass 1 only)
// 12) Check Clock status Register to verify system clock from K7 board and the 
//     interenal PLLs of JESD204B IP are locked  
// 13) Read JESD204B core register 0x000 to get IP version
// 14) Write JESD204B core register 32'h00000001 @0x008 ILA enable
// 15) Write JESD204B core register 32'h00000000 @0x00C scrambling disable
// 16) Write JESD204B core register 32'h00000001 @0x010 only responce first sysref
//     pulse (SubClass 1 only)
// 17) Write JESD204B core register 32'h00000000 @0x018 normal operation
// 18) Write JESD204B core register 32'h00000001 @0x020 Octets per Frame F = 2
// 19) Write JESD204B core register 32'h00000009 @0x024 Frames per Multiframe K = 10
// 20) Write JESD204B core register 32'h00000003 @0x028 4 lines
// 21) Write JESD204B core register 32'h00000001 @0x02C SubClass 1
// 22) Write JESD204B core register 32'h00000000 @0x030 Rx buffer delay
// 23) Write JESD204B core register 32'h00000000 @0x034 Error reporting via ~sync 
// 24) Write JESD204B core register 32'h00000001 @0x004 reset the core
// 25) Read JESD204B core register 0x004 to check if reset complete
// 26) Read JESD204B core register 0x038 to check if sync complete
// 27) Check Capture status register bit "CHVLID" to verify if JESD204B data outputs 
//     are valid, if not goto 24)
// 28) Set buffer size, channel number (channel number = 0x100) 
//     by Capture Configuration Register
// 29) Start to capture data to buffer 
//     by Capture Control Register (FETONLY = 0)
// 30) Check Capture status register to verify if buffer was full and there is 
//     no abnomal toggling of sync signal 
// 31) Configure FTDI as Sync FIFO mode
// 32) Pull data (channel 0 & 1) 
//     from FTDI fifo untill reach the set level
// 33) Configure FTDI as MPSSE mode
// 34) Reset capture engine by Capture Reset Register 
// 35) Set buffer size, channel number = 0x101 
//     by Capture Configuration Register
// 36) Start to capture data to buffer 
//     by Capture Control Register (FETONLY = 1)
// 37) Check Capture status register to verify if buffer was full and there is 
//     no abnomal toggling of sync signal 
// 38) Configure FTDI as Sync FIFO mode
// 39) Pull data channel 2 & 3 from FTDI fifo untill reach the set level
// 40) Configure FTDI as MPSSE mode
// 41) Reset capture engine by Capture Reset Register



//------------ Write JESD204B core 32 bits Register by FTDI ------------------
//  <1> write 1st(low) byte of the configuration word to JESD204B WB0 Register
//  <2> write 2nd byte of the configuration word to JESD204B WB1 Register
//  <3> write 3rd byte of the configuration word to JESD204B WB2 Register
//  <4> write 4th(high) byte of the configuration word to JESD204B WB3 Register
//  <5> wirte the high 6 bits core register address to W2INDEX of 
//      JESD204B W2index Register  
//  <6> wirte the low 6 bits core register address to WINDEX and set WRCE to "1" of 
//      JESD204B Configuration Register
//  <7> check if bit "WRACK" of JESD204B Configuration Register is "1". It means 
//      the writing is successful
//----------------------------------------------------------------------------

//------------ Read JESD204B core 32 bits Register by FTDI -------------------
//  <1> wirte the high 6 bits core register address to R2INDEX of 
//      JESD204B_R2index Register
//  <2> wirte the low 6 bits core register address to RINDEX and set RDCE to "1" of
//      JESD204B Check Register
//  <3> check if bit "RDACK" of JESD204B Check Register is "1". It means that 
//      the reading is successful
//  <4> read 1st(low) byte of the core register value from JESD204B RB0 Register
//  <5> read 2nd byte of the core register value from JESD204B RB1 Register
//  <6> read 3rd byte of the core register value from JESD204B RB2 Register
//  <7> read 4th(high) byte of the core register value from JESD204B RB3 Register
//---------------------------------------------------------------------------

            
`timescale 10ps / 1ps//gyu match timescale

//gyu definition
//`define FPGA_POSTPAR_SIM

module demo_tb ;

parameter   TOTAL_SIM_TIME   =   90000000;

//gyu ftdi
parameter   FIFOCLK_PERIOD   =   1666.6667; //60M
parameter   SYSCLK_PERIOD    =   500.00000; //200M
parameter   SPICLK_PERIOD    =   5000.0000; //20M

parameter   PCB_DELAY        =   170.5;     //1.705ns

parameter   I2C_ADDRESS      =   7'h45;
parameter   I2C_RD_DATA      =   8'ha9;

  wire           actpls;
  wire           hostrw;
  wire           adflag;
  wire           cfgrst;

  wire [7:0]     adport;

  wire           wrn_gen;
  wire           wrn;

  wire           txen_gen;
  wire           txen;

  wire [7:0]     fifo_din_gen;
  wire [7:0]     fifo_din;
  
  wire           fifoclk_gen;
  wire           fifoclk;

//gyu i2c master interface
  wire           ssda;
  reg            ssda_gen;
  wire           sscl; 

//gyu spi
  reg            sck;
  wire           scs;
  reg            mosi;
  wire           miso;

  wire           adcck;
  
  wire           adccsa;
  wire           adccsb;
  wire           clkgencs;
  wire           sysrefcs;
   
  wire           adcdi;
  reg            adcdo;

  reg            PWR_RESET;
  reg            BTN_RESET;
    
  wire   [3:0]    rxn;//gyu
  wire   [3:0]    rxp;//gyu
  
  wire            ser_DAT_A0;//gyu
  wire            ser_DAT_B0;//gyu
  wire            ser_DAT_A1;//gyu
  wire            ser_DAT_B1;//gyu

  wire           rx_sync;

  wire            ser1_DAT_A0;//gyu
  wire            ser1_DAT_B0;//gyu
  wire            ser1_DAT_A1;//gyu
  wire            ser1_DAT_B1;//gyu

  wire           rx1_sync;
    
  wire           rx_aresetn;
    
  

    //----------------------------------------------------------------------
    //----------------------- Clock Generator gyu clk ----------------------
    //----------------------------------------------------------------------
  
//system clk gen
  reg            rx_sysclk_p;//gyu clk tree
  reg            rx_sysclk_n;//gyu clk tree

  initial //gyu clk tree
  begin
     rx_sysclk_p <= #1 1'b0;
     rx_sysclk_n <= #1 1'b1;
     forever
     begin
        rx_sysclk_p <= #1 1'b1;
        rx_sysclk_n <= #1 1'b0;
        #(SYSCLK_PERIOD/2);
        rx_sysclk_p <= #1 1'b0;
        rx_sysclk_n <= #1 1'b1;
        #(SYSCLK_PERIOD/2);
     end
  end
  
//reference clk gen
wire      refclk;
wire      refclk_p;
wire      refclk_n;
wire      refclk_ctrl;

//sysref gen
wire      rx_sysref;
wire      rx_sysref_p;
wire      rx_sysref_n;
wire      rx_sysref_ctrl;
wire      rx_sysref_en;


//gyu rclk sref ctrl 
wire      rclk_sync;
wire      sref_sync;

assign #1 refclk_ctrl    = rclk_sync;
assign #1 rx_sysref_ctrl = sref_sync;


sqclkb sqclkb_refclk(.CK(demo_tb.sim_ltc2123_jesd204b_logic_v.vectors.dev_clk), .E(refclk_ctrl),     .QCK(refclk));

reg refclk_gen;
always @ (posedge refclk or negedge refclk_ctrl)
   if (~refclk_ctrl)
      refclk_gen <= #1 1'b0;
   else   
      refclk_gen <= #1 ~refclk_gen;
 
OBUFDS OBUFDS_refclk
(
.O(refclk_p),
.OB(refclk_n),

   .I(refclk_gen) //gyu v6p0   
);

sqclkb sqclkb_sysref(.CK(demo_tb.sim_ltc2123_jesd204b_logic_v.vectors.sysref),  .E(rx_sysref_ctrl),  .QCK(rx_sysref));

OBUFDS OBUFDS_rx_sysref
(
.O(rx_sysref_p),
.OB(rx_sysref_n),
.I(rx_sysref)
);


//sync gen

IBUFDS IBUFDS_rx_sync
(
.I(rx_sync_p),
.IB(rx_sync_n),
.O(rx_sync)
);


IBUFDS IBUFDS_rx1_sync
(
.I(rx1_sync_p),
.IB(rx1_sync_n),
.O(rx1_sync)
);

//spi clk gen
  initial
  begin
    sck           <= #1 1'b0;
    forever
    begin
      sck         <= #1 1'b0;
      #(SPICLK_PERIOD/2);
      sck         <= #1 1'b1;
      #(SPICLK_PERIOD/2);      
    end    
  end
  
    //----------------------------------------------------------------------
    //------------------------ Reset Generate ------------------------------
    //----------------------------------------------------------------------
  
  
//power on reset high active  
  initial
  begin
     $display("power on reset");
     PWR_RESET <= #1 1;
     #20000;
     PWR_RESET <= #1 0;  
  end

  
//push button reset high active   
  initial
  begin
     $display("push button reset");
     BTN_RESET <= #1 0;
     #(TOTAL_SIM_TIME - 999000);
     BTN_RESET <= #1 1;
     #20000;
     BTN_RESET <= #1 0;  
  end
    
 
// Global Set/Reset //just for simulation
  reg         gsr_r;
  reg         gts_r;

  assign glbl.GSR = gsr_r;
  assign glbl.GTS = gts_r;

  initial
    begin
      gts_r = 1'b0;
      gsr_r = 1'b1;
      #10000;
      gsr_r = 1'b0;
  end

  
  
      
  
  
    //----------------------------------------------------------------------
    //--------------- FTDI Config and Simlaution Module --------------------
    //----------------------------------------------------------------------  

//gyu ftdi    
pulldown(actpls);
pulldown(hostrw);
pulldown(adflag);
pullup(cfgrst);

pullup(txen_gen);

assign #PCB_DELAY wrn       = wrn_gen;    
assign #PCB_DELAY txen      = txen_gen; 
assign #PCB_DELAY fifoclk   = fifoclk_gen;
assign #PCB_DELAY fifo_din  = fifo_din_gen;
wire [31:0]   win_1st_word;
wire [31:0]   win_2nd_word;
wire [31:0]   win_3td_word;
wire [31:0]   win_4th_word;
    
wire [4:0]   K_;//gyu pass parameter

ftdi_sim #
(
.FIFOCLK_PERIOD(FIFOCLK_PERIOD),
.I2C_ADDRESS(I2C_ADDRESS)
)
ftdi_sim
(
.K_(K_),//gyu pass parameter

.PWR_RESET(PWR_RESET),

.rx_sysref_en(rx_sysref_en),

.sck(sck),
.scs(scs),
.win_1st_word(win_1st_word),
.win_2nd_word(win_2nd_word),
.win_3td_word(win_3td_word),
.win_4th_word(win_4th_word),

.ACT(actpls),
.RpWn(hostrw),
.ApDn(adflag),
.RST(cfgrst),

.ADhp(adport),

.wrn_in(wrn),
.txen_gen(txen_gen),
.fifo_din(fifo_din),
.fifoclk_gen(fifoclk_gen)

);

    //----------------------------------------------------------------------
    //----------------------- I2C Simlaution gyu i2c------------------------
    //----------------------------------------------------------------------  

//gyu i2c
pullup(ssda);
pullup(sscl); 

assign #4.5 ssda = (~ssda_gen) ? 1'b0 : 1'bz;

reg [7:0] i2c_cnt;
reg       i2c_err;

always @ (negedge sscl or posedge PWR_RESET)
  if (PWR_RESET)
  begin
    ssda_gen <= #1 1'b1;
    i2c_err  <= #1 1'b0;
    i2c_cnt  <= #1 8'b0;
  end
  else if (!i2c_err)
    casex ({i2c_cnt})
    8'h00 : if (ssda == 1'b0)//start condition
            begin
              ssda_gen <= #1 1'b1;
              i2c_cnt  <= #1 i2c_cnt + 1;
              i2c_err  <= #1 1'b0;
            end
            else
            begin
              ssda_gen <= #1 1'b1;
              i2c_cnt  <= #1 i2c_cnt + 1;
              i2c_err  <= #1 1'b1;
            end
    8'h01 : if (ssda == I2C_ADDRESS[6])
            begin
              ssda_gen <= #1 1'b1;
              i2c_cnt  <= #1 i2c_cnt + 1;
              i2c_err  <= #1 1'b0;
            end
            else
            begin
              ssda_gen <= #1 1'b1;
              i2c_cnt  <= #1 i2c_cnt + 1;
              i2c_err  <= #1 1'b1;
            end   
    8'h02 : if (ssda == I2C_ADDRESS[5])
            begin
              ssda_gen <= #1 1'b1;
              i2c_cnt  <= #1 i2c_cnt + 1;
              i2c_err  <= #1 1'b0;
            end
            else
            begin
              ssda_gen <= #1 1'b1;
              i2c_cnt  <= #1 i2c_cnt + 1;
              i2c_err  <= #1 1'b1;
            end
    8'h03 : if (ssda == I2C_ADDRESS[4])
            begin
              ssda_gen <= #1 1'b1;
              i2c_cnt  <= #1 i2c_cnt + 1;
              i2c_err  <= #1 1'b0;
            end
            else
            begin
              ssda_gen <= #1 1'b1;
              i2c_cnt  <= #1 i2c_cnt + 1;
              i2c_err  <= #1 1'b1;
            end
    8'h04 : if (ssda == I2C_ADDRESS[3])
            begin
              ssda_gen <= #1 1'b1;
              i2c_cnt  <= #1 i2c_cnt + 1;
              i2c_err  <= #1 1'b0;
            end
            else
            begin
              ssda_gen <= #1 1'b1;
              i2c_cnt  <= #1 i2c_cnt + 1;
              i2c_err  <= #1 1'b1;
            end
    8'h05 : if (ssda == I2C_ADDRESS[2])
            begin
              ssda_gen <= #1 1'b1;
              i2c_cnt  <= #1 i2c_cnt + 1;
              i2c_err  <= #1 1'b0;
            end
            else
            begin
              ssda_gen <= #1 1'b1;
              i2c_cnt  <= #1 i2c_cnt + 1;
              i2c_err  <= #1 1'b1;
            end                           
    8'h06 : if (ssda == I2C_ADDRESS[1])
            begin
              ssda_gen <= #1 1'b1;
              i2c_cnt  <= #1 i2c_cnt + 1;
              i2c_err  <= #1 1'b0;
            end
            else
            begin
              ssda_gen <= #1 1'b1;
              i2c_cnt  <= #1 i2c_cnt + 1;
              i2c_err  <= #1 1'b1;
            end
    8'h07 : if (ssda == I2C_ADDRESS[0])
            begin
              ssda_gen <= #1 1'b1;
              i2c_cnt  <= #1 i2c_cnt + 1;
              i2c_err  <= #1 1'b0;
            end
            else
            begin
              ssda_gen <= #1 1'b1;
              i2c_cnt  <= #1 i2c_cnt + 1;
              i2c_err  <= #1 1'b1;
            end
    8'h08 : if (ssda == 1'b1)//read flag and issue ack
            begin
              ssda_gen <= #1 1'b0;
              i2c_cnt  <= #1 i2c_cnt + 1;
              i2c_err  <= #1 1'b0;
            end
            else
            begin
              ssda_gen <= #1 1'b1;
              i2c_cnt  <= #1 i2c_cnt + 1;
              i2c_err  <= #1 1'b1;
            end  
    8'h09 : begin
              ssda_gen <= #1 I2C_RD_DATA[7];
              i2c_cnt  <= #1 i2c_cnt + 1;
              end
    8'h0a : begin
              ssda_gen <= #1 I2C_RD_DATA[6];
              i2c_cnt  <= #1 i2c_cnt + 1;
            end
    8'h0b : begin
              ssda_gen <= #1 I2C_RD_DATA[5];
              i2c_cnt  <= #1 i2c_cnt + 1;
            end  
    8'h0c : begin
              ssda_gen <= #1 I2C_RD_DATA[4];
              i2c_cnt  <= #1 i2c_cnt + 1;
            end                                           
    8'h0d : begin
              ssda_gen <= #1 I2C_RD_DATA[3];
              i2c_cnt  <= #1 i2c_cnt + 1;
            end
    8'h0e : begin
              ssda_gen <= #1 I2C_RD_DATA[2];
              i2c_cnt  <= #1 i2c_cnt + 1;
            end    
    8'h0f : begin
              ssda_gen <= #1 I2C_RD_DATA[1];
              i2c_cnt  <= #1 i2c_cnt + 1;
            end
    8'h10 : begin
              ssda_gen <= #1 I2C_RD_DATA[0];
              i2c_cnt  <= #1 i2c_cnt + 1;
            end
    8'h11 : begin
              ssda_gen <= #1 1'b1;
              i2c_cnt  <= #1 i2c_cnt + 1;
            end         
    8'h12 : if (ssda == 1'b0)//read ack
            begin
              ssda_gen <= #1 1'b1;
              i2c_cnt  <= #1 8'h0;
              i2c_err  <= #1 1'b0;
            end
            else
            begin
              ssda_gen <= #1 1'b1;
              i2c_cnt  <= #1 i2c_cnt;
              i2c_err  <= #1 1'b1;
            end
     endcase
            
   
            
    //----------------------------------------------------------------------
    //-------------------- Config ADC By SPI gyu spi -----------------------
    //----------------------------------------------------------------------
    
      
  reg  [127:0]  win_gen_reg;
  reg  [7:0]    win_count;  
  always @(negedge sck or posedge scs)
    if (scs)
    begin
      mosi           <= #1 1'b1;
      win_gen_reg    <= #1 {win_1st_word, win_2nd_word, win_3td_word, win_4th_word};
      win_count      <= #1 8'b0;
    end
    else if (win_count == 8'h80)
    begin
      mosi           <= #1 1'b1;
      win_gen_reg    <= #1 win_gen_reg;
      win_count      <= #1 win_count;
    end
    else
    begin
      mosi           <= #1 win_gen_reg[127];
      win_gen_reg    <= #1 win_gen_reg << 1;
      win_count      <= #1 win_count + 1;
    end  
  
    //----------------------------------------------------------------------
    //------------------------------ adcdo gen -----------------------------
    //----------------------------------------------------------------------
    always @(negedge sck or posedge scs)
      if (scs)
        adcdo        <= #1 1'b0;                  
      else
        adcdo        <= #1 ~sck;                   
       
    
    
    //----------------------------------------------------------------------
    //------------------- LTC2123 simulation module ------------------------
    //----------------------------------------------------------------------

        
  assign rxn[0]   = ~ser_DAT_A0;//gyu
  assign rxp[0]   = ser_DAT_A0;//gyu
  
  assign rxn[1]   = ~ser_DAT_B0;//gyu
  assign rxp[1]   = ser_DAT_B0;//gyu
    
  assign rxn[2]   = ~ser1_DAT_A0;//gyu
  assign rxp[2]   = ser1_DAT_A0;//gyu
  
  assign rxn[3]   = ~ser1_DAT_B0;//gyu
  assign rxp[3]   = ser1_DAT_B0;//gyu

  
sim_ltc2123_jesd204b_logic_v #
(
   .DATA_A_PLUS (8'h00),
   .DATA_B_PLUS (8'h00)
)
sim_ltc2123_jesd204b_logic_v(
  .K_(K_),//gyu pass parameter
  .sync(rx_sync),//gyu handled by FPGA
  .div2_rstb(),
  .PLL_ref(),
  .DAT_A1(),
  .DAT_A0(),
  .DAT_B0(),
  .DAT_B1(),
  .ser_DAT_A0(ser_DAT_A0),
  .ser_DAT_B0(ser_DAT_B0),
  .ser_DAT_A1(ser_DAT_A1),
  .ser_DAT_B1(ser_DAT_B1),
  .ser_A1_en(),
  .ser_A0_en(),
  .ser_B0_en(),
  .ser_B1_en(),
  .two_adc_per_ln(),
  .two_ln_per_adc()

);

sim_ltc2123_jesd204b_logic_v #
(
   //.DATA_A_PLUS (8'h13),
   //.DATA_B_PLUS (8'h31),
   .DATA_A_PLUS (8'h00),
   .DATA_B_PLUS (8'h00),
   .SYSREF_SHIFT (4'h7)
)
sim_ltc2123_jesd204b_logic_v1(
  .sync(rx1_sync),//gyu handled by FPGA
  .div2_rstb(),
  .PLL_ref(),
  .DAT_A1(),
  .DAT_A0(),
  .DAT_B0(),
  .DAT_B1(),
  .ser_DAT_A0(ser1_DAT_A0),
  .ser_DAT_B0(ser1_DAT_B0),
  .ser_DAT_A1(ser1_DAT_A1),
  .ser_DAT_B1(ser1_DAT_B1),
  .ser_A1_en(),
  .ser_A0_en(),
  .ser_B0_en(),
  .ser_B1_en(),
  .two_adc_per_ln(),
  .two_ln_per_adc()

);

    //----------------------------------------------------------------------
    //---------------------------- FPGA module -----------------------------
    //----------------------------------------------------------------------
    
    
jd204bv6p0l4sc0_example_design DUT(

//gyu FTDI Sync FIFO interface
   .fifo_data              (fifo_din_gen),
   .txen                   (txen),
   .wrn                    (wrn_gen),
   .fifoclk                (fifoclk),
 
//gyu  General Control and Jumper
   .actpls                 (actpls),
   .hostrw                 (hostrw),
   .adflag                 (adflag),
   .cfgrst                 (cfgrst),

   .adport                 (adport),

//gyu i2c master interface
   .ssda                   (ssda),
   .sscl                   (sscl), 

//gyu spi master input from FDTI
   .sck                    (sck),
   .scs                    (scs),
   .mosi                   (mosi),
   .miso                   (miso),

//gyu spi slave output to ADC
   .adcck                  (adcck),
   .adccsa                 (adccs),
   .adccsb                 (adccsb),
   .clkgencs               (clkgencs),
   .sysrefcs               (sysrefcs),
   .adcdi                  (adcdi),
   .adcdo                  (adcdo),
   
//gyu rclk sref ctrl   
   .rclk_sync              (rclk_sync),
   .sref_sync              (sref_sync),
   
   .refclk0p               (refclk_p),
   .refclk0n               (refclk_n),

    
   .rx_reset               (PWR_RESET | BTN_RESET),

   

   .rxp(rxp[3:0]),
   .rxn(rxn[3:0]),
   
   .rx_sysref_p            (rx_sysref_p),
   .rx_sysref_n            (rx_sysref_n),


   .rx_sysclk_p            (rx_sysclk_p),//gyu clk tree
   .rx_sysclk_n            (rx_sysclk_n),//gyu clk tree


// Rx AXI common signals
   .rx_aresetn             (rx_aresetn),
  

   .rx1_sync_p              (rx1_sync_p),
   .rx1_sync_n              (rx1_sync_n),
 
   
   .rx_sync_p              (rx_sync_p),
   .rx_sync_n              (rx_sync_n)
   

);



    //----------------------------------------------------------------------
    //------------------------- Simulation Dumpfile ------------------------
    //----------------------------------------------------------------------


  // Check for runaway simulation
  initial
    begin
      //gyu create fsdb
      `ifdef FPGA_POSTPAR_SIM
                                      
                $sdf_annotate("../jd204bv6p0l4sc0/jd204bv6p0l4sc0_example/jd204bv6p0l4sc0_example.sim/sim_1/impl/timing/demo_tb_time_impl.sdf",demo_tb.DUT, ,"sdfbase.log","MAXIMUM");
                            
                $fsdbDumpfile ("demo_tb_post.fsdb");
      `else
                $fsdbDumpfile ("demo_tb_behav.fsdb");   
      `endif           
                $fsdbDumpvars (0,demo_tb);
                $fsdbDumpon;
      //gyu create fsdb
      
      $display("Test Started");
      #TOTAL_SIM_TIME;
      
      $finish;
    end // initial begin
   
endmodule //testbench
